class grabinPRI extends xPlayerReplicationInfo;

#exec AUDIO IMPORT FILE="gesture.WAV" NAME="gesture" GROUP="gestureFX"
#exec AUDIO IMPORT FILE="cast.WAV" NAME="cast" GROUP="gestureFX"

//this class essentially controls pawn-replicated gesturing functions.
//non-replicated functions are present in gestureInventory.uc

var string debugStr;

var bool stoppedFire; //pass to HUD for xhair fade effect

//=================
// to add a new gesture:
// - give it a string var and declare it in defaults
// - put its IF statement in interp()
// - write its execution function
// - put the function name in the replication statement
//=================

//VARS FOR WEAPON TYPE STUFF
var class<Projectile> ProjectileClass;
var Vector ProjSpawnOffset; // +x forward, +y right, +z up
var int ProjPerFire;
var float DamageAtten; // attenuate instant-hit/projectile damage by this multiplier
var float Spread; // rotator units. spread of SS_Random proj's
var enum ESpreadStyle
{
    SS_None,
    SS_Random, // spread is max random angle deviation
    SS_Line,   // spread is angle between each projectile
    SS_Ring
} SpreadStyle;

//VARS FOR GESTURE IDs FOR DIFFERENT THINGS
var() string fireballStr, energyStr, bigEnergyStr, qbeamStr;

replication
{
	reliable if (ROLE < ROLE_Authority)
		drawEffect, drawCursor, randomiseFX, playRelease, playAmbient,
		DoProjectile, setDefProj,
		fireball, energyBolts, bigEnergy, quickBeam;
}

/*function postNetBeginPlay()
{
	Local Actor A;
	
	Super.PostNetBeginPlay();
	ForEach DynamicActors(class'Actor', A)
	{
		// find my controller and assign it
		if ( grabinController(A) != None && grabinController(A).PlayerReplicationInfo == self )
		{
			myPC = grabinController(A);
			break;
		}
	}
	log("ITS A " $ myPC);
}*/

//================================================
//===MAIN FUNCTIONS FOR GESTUREid RESPONSE CALC===
//================================================

simulated function bool interp(array<int> gestureID)
{
	local int i;
	local string strID;
		//this is kind of dumb, but better accessible. It stores the ID array
		// as an easy, non-associative datatype. Stops us needing many iterators.
	
	for ( i=0; i < gestureID.Length; i++ )
	{  //converts to string
		strID = strID $ gestureID[i];
	}
	
	debugStr = strID;	//FOR DEBUGGING
	
	setDefProj();
	if ( strID == fireballStr )
		fireBall();
	else if ( strID == energyStr )
		energyBolts();
	else if ( strID == bigEnergyStr )
		bigEnergy();
	else if ( strID == qbeamStr )
		quickBeam();
	
	return true;
}

//================================================
//===========DIFFERENT GESTURE FUNCTIONS==========
//================================================

function setDefProj()
{  //makes setting the values for firing 1 projectile easier
	ProjSpawnOffset.X = 35;
	ProjPerFire = 1;
	DamageAtten = 1.0;
	SpreadStyle=SS_None;
	Spread = 0.0;
}

//============
// basic attacks
//============

function fireBall()
{
	ProjectileClass = Class'skaarjPack.GasBagBelch';
	grabinPawn(Controller(Owner).Pawn).setGestureState('holdProjAttack');
}

function energyBolts()
{
	ProjectileClass = Class'skaarjPack.KrallBolt';
	ProjPerFire = 3;
	SpreadStyle = SS_Random;
	Spread = 4000;
	grabinPawn(Controller(Owner).Pawn).setGestureState('holdProjAttack');
}

function bigEnergy()
{
	ProjectileClass = Class'onslaughtFull.ONSMASRocketProjectile';
	grabinPawn(Controller(Owner).Pawn).setGestureState('holdProjAttack');
}

function quickBeam()
{
	ProjectileClass = Class'ut2k4Assault.PROJ_Sentinel_Laser';
	grabinPawn(Controller(Owner).Pawn).setGestureState('holdProjAttack');
}

//================================================
//====THESE FUNCTIONS SERVERSIDE TO SPAWN FX======
//================================================

simulated function drawEffect(xEmitter gEffect, vector spawnTo, vector playerCamLoc)
{
		//spawn effect at world center (effect should have warmup time)
	gEffect = spawn(Class'gestureDrawFX',self,,spawnTo,);
   		//move effect to absolute coords
   	gEffect.Move(gEffect.Location + playerCamLoc);
   	gEffect.bUpdateSimulatedPosition = True;
   	randomiseFX(gEffect);
}

simulated function drawCursor(xEmitter hEffect, vector spawnTo, vector playerCamLoc)
{
	hEffect = spawn(Class'gestureCursorFX',self,,spawnTo,);
	hEffect.Move(hEffect.Location + playerCamLoc);
	hEffect.bUpdateSimulatedPosition = True;
}

simulated function randomiseFX(xEmitter gEffect)
{
	//this function just randomises the gesture particle FX a little in color and size
	if (rand(10) == 1)
   	{
   		gEffect.mSizeRange[0]=2.000000;
   		gEffect.mSizeRange[1]=5.000000;
   	}
   	if (rand(4) == 1)
   		gEffect.skins[0]=Texture'EmitterTextures.Flares.EflareP2';
}

//================================================
//==FUNCTIONS TO PLAY SOUNDS ON THE PLAYER'S PAWN=
//================================================

simulated function playRelease()
{ //chooses a casting sound and plays it
	Controller(Owner).Pawn.PlaySound(Sound'gestureFX.cast', SLOT_None,255,,128,,true);
}

simulated function playAmbient(bool on)
{ //sets the ambient effect for gesturing
	if (on)
		Controller(Owner).Pawn.AmbientSound = Sound'gestureFX.gesture';
	else
		Controller(Owner).Pawn.AmbientSound = None;
}

//================================================
//============WEAPON ATTACK FUNCTIONS=============
//================================================

function DoProjectile()
{
    local Vector StartProj, StartTrace, X,Y,Z;
    local Rotator R, Aim;
    local Vector HitLocation, HitNormal;
    local Actor Other;
    local int p;
    local int SpawnCount;
    local float theta;

    Controller(Owner).Pawn.MakeNoise(1.0);
    GetViewAxes(X,Y,Z);

    StartTrace = Controller(Owner).Pawn.Location + Controller(Owner).Pawn.EyePosition();
      // + X*Controller(Owner).Pawn.CollisionRadius;
    StartProj = StartTrace + X*ProjSpawnOffset.X;

    // check if projectile would spawn through a wall and adjust start location accordingly
    Other = Trace(HitLocation, HitNormal, StartProj, StartTrace, false);
    if (Other != None)
    {
        StartProj = HitLocation;
    }
    
    Aim = Controller(Owner).Rotation;

    SpawnCount = Max(1, ProjPerFire);

    switch (SpreadStyle)
    {
    case SS_Random:
        X = Vector(Aim);
        for (p = 0; p < SpawnCount; p++)
        {
            R.Yaw = Spread * (FRand()-0.5);
            R.Pitch = Spread * (FRand()-0.5);
            R.Roll = Spread * (FRand()-0.5);
            SpawnProjectile(StartProj, Rotator(X >> R));
        }
        break;
    case SS_Line:
        for (p = 0; p < SpawnCount; p++)
        {
            theta = Spread*PI/32768*(p - float(SpawnCount-1)/2.0);
            X.X = Cos(theta);
            X.Y = Sin(theta);
            X.Z = 0.0;
            SpawnProjectile(StartProj, Rotator(X >> Aim));
        }
        break;
    default:
        SpawnProjectile(StartProj, Aim);
    }
}

function projectile SpawnProjectile(Vector Start, Rotator Dir)
{
    local Projectile p;
    
    if( ProjectileClass != None )
        p = Spawn(ProjectileClass,,, Start, Dir);

    if( p == None )
        return None;

    p.Damage *= DamageAtten;
    stoppedFire = True;
    return p;
}

simulated function GetViewAxes( out vector xaxis, out vector yaxis, out vector zaxis )
{
        GetAxes( Controller(Owner).Rotation, xaxis, yaxis, zaxis );
}

//================================================
//==================DEFAULTS======================
//================================================

defaultproperties
{
	stoppedFire=false;
	ProjectileClass=none
	ProjPerFire=1
	DamageAtten=1.0
	SpreadStyle=SS_None
	Spread=0.0
	fireballStr="018765"
	energyStr="07272"
	bigEnergyStr="0123456781082"
	qbeamStr="0567175"
}
